import random
import runpy
import sys

import mockturtle

sys.modules['turtle'] = sys.modules['mockturtle']


def test_fidget():
    random.seed(0)
    mockturtle.events[:] = (
        [('timer',), ('key space',)] * 30
        + [('timer',)] * 600
    )
    runpy.run_module('freegames.fidget')



import random
import runpy
import sys

import mockturtle

sys.modules['turtle'] = sys.modules['mockturtle']


def test_connect():
    random.seed(0)
    mockturtle.events[:] = [
        ('click', 25, 0),
        ('click', 75, 0),
    ]
    runpy.run_module('freegames.connect')



import random
import runpy
import sys

import mockturtle

sys.modules['turtle'] = sys.modules['mockturtle']


def test_cannon():
    random.seed(0)
    mockturtle.events[:] = (
        [('timer',)] * 300
        + [('click', 0, 0)]
        + [('timer', True)] * 3000
    )
    runpy.run_module('freegames.cannon')



import random
import runpy
import unittest.mock as mock


def test_guess():
    random.seed(0)
    mock_input = mock.Mock()
    mock_input.side_effect = [20, 70, 50]
    mocks = {'print': lambda *args: None, 'input': mock_input}

    with mock.patch.multiple('builtins', **mocks):
        runpy.run_module('freegames.guess')



import random
import runpy
import sys

import mockturtle

sys.modules['turtle'] = sys.modules['mockturtle']


def test_flappy_outside():
    random.seed(0)
    mockturtle.events[:] = (
        [('timer', True)] * 300
    )
    runpy.run_module('freegames.flappy')


def test_flappy_collision():
    random.seed(0)
    mockturtle.events[:] = (
        ([('timer', True)] * 6 + [('click', 0, 0)]) * 100
    )
    runpy.run_module('freegames.flappy')



import random
import runpy
import sys

import mockturtle

sys.modules['turtle'] = sys.modules['mockturtle']


def test_memory():
    random.seed(0)
    events = []

    for x in range(-200, 200, 50):
        for y in range(-200, 200, 50):
            events += [('timer',)] * 10
            events += [('click', x + 25, y + 25)]

    events += [('timer',)] * 10
    mockturtle.events[:] = events
    runpy.run_module('freegames.memory')



import random
import runpy
import sys

import mockturtle

sys.modules['turtle'] = sys.modules['mockturtle']


def test_tron_1():
    random.seed(0)
    mockturtle.events[:] = (
        [('key a',), ('key a',)]
        + [('timer', True)] * 600
    )
    runpy.run_module('freegames.tron')

def test_tron_2():
    random.seed(0)
    mockturtle.events[:] = (
        [('key j',), ('key j',)]
        + [('timer', True)] * 600
    )
    runpy.run_module('freegames.tron')



from pytest import raises

import freegames.utils as utils

def test_change_after_hash():
    v = utils.vector(0, 0)
    hash(v)
    with raises(ValueError):
        v.x = 1
    with raises(ValueError):
        v.y = 1
    with raises(ValueError):
        v += 1
    with raises(ValueError):
        v -= 1
    with raises(ValueError):
        v *= 2
    with raises(ValueError):
        v /= 2
    with raises(ValueError):
        v.rotate(90)

def test_not_implemented_paths():
    v = utils.vector(0, 0)
    assert not (v == 0)
    assert v != 0



"""Flappy, game inspired by Flappy Bird.

Exercises

1. Keep score.
2. Vary the speed.
3. Vary the size of the balls.
4. Allow the bird to move forward and back.

"""

from random import *
from turtle import *
from freegames import vector

bird = vector(0, 0)
balls = []

def tap(x, y):
    "Move bird up in response to screen tap."
    up = vector(0, 30)
    bird.move(up)

def inside(point):
    "Return True if point on screen."
    return -200 < point.x < 200 and -200 < point.y < 200

def draw(alive):
    "Draw screen objects."
    clear()

    goto(bird.x, bird.y)

    if alive:
        dot(10, 'green')
    else:
        dot(10, 'red')

    for ball in balls:
        goto(ball.x, ball.y)
        dot(20, 'black')

    update()

def move():
    "Update object positions."
    bird.y -= 5

    for ball in balls:
        ball.x -= 3

    if randrange(10) == 0:
        y = randrange(-199, 199)
        ball = vector(199, y)
        balls.append(ball)

    while len(balls) > 0 and not inside(balls[0]):
        balls.pop(0)

    if not inside(bird):
        draw(False)
        return

    for ball in balls:
        if abs(ball - bird) < 15:
            draw(False)
            return

    draw(True)
    ontimer(move, 50)

setup(420, 420, 370, 0)
hideturtle()
up()
tracer(False)
onscreenclick(tap)
move()
done()



import random
import runpy
import sys
import unittest.mock as mock

import mockturtle

sys.modules['turtle'] = sys.modules['mockturtle']


def test_simonsays():
    random.seed(0)
    mockturtle.events[:] = (
        ('click', 0, 0),
        ('click', -100, -100),
        ('click', 100, 100),
    )

    try:
        with mock.patch('time.sleep', lambda delay: None):
            runpy.run_module('freegames.simonsays')
    except SystemExit:
        pass



import random
import runpy
import sys

import mockturtle

sys.modules['turtle'] = sys.modules['mockturtle']


def test_ant():
    random.seed(0)
    mockturtle.events[:] = [('timer',)] * 600
    runpy.run_module('freegames.ant')



"""Free Python Games

Free Python Games is an Apache2 licensed collection of free Python games
intended for education and fun. The games are written in simple Python code and
designed for experimentation and changes. Simplified versions of several
classic arcade games are included.

Python is one of the top-five most popular programming languages in the world
and available for free from www.python.org. Python includes an extensive
Standard Library distributed with your installation. The Standard Library has a
module called Turtle which is a popular way to introduce programming to
kids. Turtle was part of the original Logo programming language developed by
Wally Feurzig and Seymour Papert in 1966. All of the games in Free Python Games
are implemented using Python and its Turtle module.

Starting in 2012, Free Python Games began as an after school program to teach
programming to inner-city youth. The goal was to have fun as much as it was to
learn. Since then the games have been improved and used in a variety of
settings ranging from classrooms to summer day-camps.

The games run anywhere Python can be installed which includes desktop computers
running Windows, Mac OS, or Linux and older or low-power hardware such as the
Raspberry Pi. Kids across the United States in grades 6th-12th have enjoyed
learning about topics such as encryption and projectile motion through games.

Each game is entirely independent from the others and includes comments along
with a list of exercises to work through with students. Creativity and
flexibility is important. There is no right or wrong way to implement a new
feature or behavior! You never know which games students will engage with best.

Free Python Games supports a command-line interface (CLI). Help for the CLI is
available using::

  $ python3 -m freegames --help

The CLI supports three commands: list, copy, and show. For a list of all games
run::

  $ python3 -m freegames list

Any of the listed games may be played by executing the Python module from the
command-line. To reference the Python module, combine "freegames" with the name
of the game. For example, to play the "snake" game run::

  $ python3 -m freegames.snake

Games can be modified by copying their source code. The copy command will
create a Python file in your local directory which you can edit. For example,
to copy and play the "snake" game run::

  $ python3 -m freegames copy snake
  $ python3 snake.py

Python includes a built-in text editor named IDLE which can also execute Python
code. To launch the editor and make changes to the "snake" game run::

  $ python3 -m idlelib.idle snake.py

"""

from .utils import floor, line, path, square, vector

__all__ = ['floor', 'line', 'path', 'square', 'vector']

__title__ = 'freegames'
__version__ = '2.0.3'
__build__ = 0x020003
__author__ = 'Grant Jenks'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2017 Grant Jenks'



import random
import runpy
import unittest.mock as mock


def test_crypto_encrypt():
    random.seed(0)
    mock_input = mock.Mock()
    mock_input.side_effect = ['encrypt', 'Python since 1991', '10']
    mocks = {'print': lambda *args: None, 'input': mock_input}

    with mock.patch.multiple('builtins', **mocks):
        runpy.run_module('freegames.crypto')

def test_crypto_encrypt_bad_key():
    random.seed(0)
    mock_input = mock.Mock()
    mock_input.side_effect = ['encrypt', 'Python since 1991', 'abc']
    mocks = {'print': lambda *args: None, 'input': mock_input}

    with mock.patch.multiple('builtins', **mocks):
        runpy.run_module('freegames.crypto')

def test_crypto_bad_command():
    random.seed(0)
    mock_input = mock.Mock()
    mock_input.side_effect = ['bad command']
    mocks = {'print': lambda *args: None, 'input': mock_input}

    with mock.patch.multiple('builtins', **mocks):
        runpy.run_module('freegames.crypto')

def test_crypto_decrypt():
    random.seed(0)
    mock_input = mock.Mock()
    mock_input.side_effect = ['decrypt', 'Zidryx csxmo 1991', '10']
    mocks = {'print': lambda *args: None, 'input': mock_input}

    with mock.patch.multiple('builtins', **mocks):
        runpy.run_module('freegames.crypto')

def test_crypto_decode():
    random.seed(0)
    mock_input = mock.Mock()
    mock_input.side_effect = ['decode', 'Zidryx csxmo 1991']
    mocks = {'print': lambda *args: None, 'input': mock_input}

    with mock.patch.multiple('builtins', **mocks):
        runpy.run_module('freegames.crypto')



import random
import runpy
import sys

import mockturtle

sys.modules['turtle'] = sys.modules['mockturtle']


def test_pacman():
    random.seed(0)
    mockturtle.events[:] = (
        [('timer', True), ('key Up',)] * 600
        + [('timer', True)] * 3000
    )
    runpy.run_module('freegames.pacman')



import random
import runpy
import sys

import mockturtle

sys.modules['turtle'] = sys.modules['mockturtle']


def test_snake():
    random.seed(0)
    mockturtle.events[:] = (
        [('timer',), ('key Left',), ('timer',), ('key Up',)]
        + [('timer', True)] * 300
    )
    runpy.run_module('freegames.snake')



"""Utilities

"""
# pylint: disable=no-member

import collections
import math
import os


def floor(value, size, offset=200):
    """Floor of `value` given `size` and `offset`.

    The floor function is best understood with a diagram of the number line::

         -200  -100    0    100   200
        <--|--x--|-----|--y--|--z--|-->

    The number line shown has offset 200 denoted by the left-hand tick mark at
    -200 and size 100 denoted by the tick marks at -100, 0, 100, and 200. The
    floor of a value is the left-hand tick mark of the range where it lies. So
    for the points show above: ``floor(x)`` is -200, ``floor(y)`` is 0, and
    ``floor(z)`` is 100.

    >>> floor(10, 100)
    0.0
    >>> floor(120, 100)
    100.0
    >>> floor(-10, 100)
    -100.0
    >>> floor(-150, 100)
    -200.0
    >>> floor(50, 167)
    -33.0

    """
    return float(((value + offset) // size) * size - offset)


def path(filename):
    "Return full path to `filename` in freegames module."
    filepath = os.path.realpath(__file__)
    dirpath = os.path.dirname(filepath)
    fullpath = os.path.join(dirpath, filename)
    return fullpath


def line(a, b, x, y):
    "Draw line from `(a, b)` to `(x, y)`."
    import turtle
    turtle.up()
    turtle.goto(a, b)
    turtle.down()
    turtle.goto(x, y)


def square(x, y, size, name):
    """Draw square at `(x, y)` with side length `size` and fill color `name`.

    The square is oriented so the bottom left corner is at (x, y).

    """
    import turtle
    turtle.up()
    turtle.goto(x, y)
    turtle.down()
    turtle.color(name)
    turtle.begin_fill()

    for count in range(4):
        turtle.forward(size)
        turtle.left(90)

    turtle.end_fill()


class vector(collections.Sequence):
    """Two-dimensional vector.

    Vectors can be modified in-place.

    >>> v = vector(0, 1)
    >>> v.move(1)
    >>> v
    vector(1, 2)
    >>> v.rotate(90)
    >>> v
    vector(-2.0, 1.0)

    """
    # pylint: disable=invalid-name
    PRECISION = 6

    __slots__ = ('_x', '_y', '_hash')

    def __init__(self, x, y):
        """Initialize vector with coordinates: x, y.

        >>> v = vector(1, 2)
        >>> v.x
        1
        >>> v.y
        2

        """
        self._hash = None
        self._x = round(x, self.PRECISION)
        self._y = round(y, self.PRECISION)

    @property
    def x(self):
        """X-axis component of vector.

        >>> v = vector(1, 2)
        >>> v.x
        1
        >>> v.x = 3
        >>> v.x
        3

        """
        return self._x

    @x.setter
    def x(self, value):
        if self._hash is not None:
            raise ValueError('cannot set x after hashing')
        self._x = round(value, self.PRECISION)

    @property
    def y(self):
        """Y-axis component of vector.

        >>> v = vector(1, 2)
        >>> v.y
        2
        >>> v.y = 5
        >>> v.y
        5

        """
        return self._y

    @y.setter
    def y(self, value):
        if self._hash is not None:
            raise ValueError('cannot set y after hashing')
        self._y = round(value, self.PRECISION)

    def __hash__(self):
        """v.__hash__() -> hash(v)

        >>> v = vector(1, 2)
        >>> h = hash(v)
        >>> v.x = 2
        Traceback (most recent call last):
            ...
        ValueError: cannot set x after hashing

        """
        if self._hash is None:
            pair = (self.x, self.y)
            self._hash = hash(pair)
        return self._hash

    def __len__(self):
        """v.__len__() -> len(v)

        >>> v = vector(1, 2)
        >>> len(v)
        2

        """
        return 2

    def __getitem__(self, index):
        """v.__getitem__(v, i) -> v[i]

        >>> v = vector(3, 4)
        >>> v[0]
        3
        >>> v[1]
        4
        >>> v[2]
        Traceback (most recent call last):
            ...
        IndexError

        """
        if index == 0:
            return self.x
        elif index == 1:
            return self.y
        else:
            raise IndexError

    def copy(self):
        """Return copy of vector.

        >>> v = vector(1, 2)
        >>> w = v.copy()
        >>> v is w
        False

        """
        type_self = type(self)
        return type_self(self.x, self.y)

    def __eq__(self, other):
        """v.__eq__(w) -> v == w

        >>> v = vector(1, 2)
        >>> w = vector(1, 2)
        >>> v == w
        True

        """
        if isinstance(other, vector):
            return self.x == other.x and self.y == other.y
        return NotImplemented

    def __ne__(self, other):
        """v.__ne__(w) -> v != w

        >>> v = vector(1, 2)
        >>> w = vector(3, 4)
        >>> v != w
        True

        """
        if isinstance(other, vector):
            return self.x != other.x or self.y != other.y
        return NotImplemented

    def __iadd__(self, other):
        """v.__iadd__(w) -> v += w

        >>> v = vector(1, 2)
        >>> w = vector(3, 4)
        >>> v += w
        >>> v
        vector(4, 6)
        >>> v += 1
        >>> v
        vector(5, 7)

        """
        if self._hash is not None:
            raise ValueError('cannot add vector after hashing')
        elif isinstance(other, vector):
            self.x += other.x
            self.y += other.y
        else:
            self.x += other
            self.y += other
        return self

    def __add__(self, other):
        """v.__add__(w) -> v + w

        >>> v = vector(1, 2)
        >>> w = vector(3, 4)
        >>> v + w
        vector(4, 6)
        >>> v + 1
        vector(2, 3)
        >>> 2.0 + v
        vector(3.0, 4.0)

        """
        copy = self.copy()
        return copy.__iadd__(other)

    __radd__ = __add__

    def move(self, other):
        """Move vector by other (in-place).

        >>> v = vector(1, 2)
        >>> w = vector(3, 4)
        >>> v.move(w)
        >>> v
        vector(4, 6)
        >>> v.move(3)
        >>> v
        vector(7, 9)

        """
        self.__iadd__(other)

    def __isub__(self, other):
        """v.__isub__(w) -> v -= w

        >>> v = vector(1, 2)
        >>> w = vector(3, 4)
        >>> v -= w
        >>> v
        vector(-2, -2)
        >>> v -= 1
        >>> v
        vector(-3, -3)

        """
        if self._hash is not None:
            raise ValueError('cannot subtract vector after hashing')
        elif isinstance(other, vector):
            self.x -= other.x
            self.y -= other.y
        else:
            self.x -= other
            self.y -= other
        return self

    def __sub__(self, other):
        """v.__sub__(w) -> v - w

        >>> v = vector(1, 2)
        >>> w = vector(3, 4)
        >>> v - w
        vector(-2, -2)
        >>> v - 1
        vector(0, 1)

        """
        copy = self.copy()
        return copy.__isub__(other)

    def __imul__(self, other):
        """v.__imul__(w) -> v *= w

        >>> v = vector(1, 2)
        >>> w = vector(3, 4)
        >>> v *= w
        >>> v
        vector(3, 8)
        >>> v *= 2
        >>> v
        vector(6, 16)

        """
        if self._hash is not None:
            raise ValueError('cannot multiply vector after hashing')
        elif isinstance(other, vector):
            self.x *= other.x
            self.y *= other.y
        else:
            self.x *= other
            self.y *= other
        return self

    def __mul__(self, other):
        """v.__mul__(w) -> v * w

        >>> v = vector(1, 2)
        >>> w = vector(3, 4)
        >>> v * w
        vector(3, 8)
        >>> v * 2
        vector(2, 4)
        >>> 3.0 * v
        vector(3.0, 6.0)

        """
        copy = self.copy()
        return copy.__imul__(other)

    __rmul__ = __mul__

    def scale(self, other):
        """Scale vector by other (in-place).

        >>> v = vector(1, 2)
        >>> w = vector(3, 4)
        >>> v.scale(w)
        >>> v
        vector(3, 8)
        >>> v.scale(0.5)
        >>> v
        vector(1.5, 4.0)

        """
        self.__imul__(other)

    def __itruediv__(self, other):
        """v.__itruediv__(w) -> v /= w

        >>> v = vector(2, 4)
        >>> w = vector(4, 8)
        >>> v /= w
        >>> v
        vector(0.5, 0.5)
        >>> v /= 2
        >>> v
        vector(0.25, 0.25)

        """
        if self._hash is not None:
            raise ValueError('cannot divide vector after hashing')
        elif isinstance(other, vector):
            self.x /= other.x
            self.y /= other.y
        else:
            self.x /= other
            self.y /= other
        return self

    def __truediv__(self, other):
        """v.__truediv__(w) -> v / w

        >>> v = vector(1, 2)
        >>> w = vector(3, 4)
        >>> w / v
        vector(3.0, 2.0)
        >>> v / 2
        vector(0.5, 1.0)

        """
        copy = self.copy()
        return copy.__itruediv__(other)

    def __neg__(self):
        """v.__neg__() -> -v

        >>> v = vector(1, 2)
        >>> -v
        vector(-1, -2)

        """
        # pylint: disable=invalid-unary-operand-type
        copy = self.copy()
        copy.x = -copy.x
        copy.y = -copy.y
        return copy

    def __abs__(self):
        """v.__abs__() -> abs(v)

        >>> v = vector(3, 4)
        >>> abs(v)
        5.0

        """
        return (self.x ** 2 + self.y ** 2) ** 0.5

    def rotate(self, angle):
        """Rotate vector counter-clockwise by angle (in-place).

        >>> v = vector(1, 2)
        >>> v.rotate(90)
        >>> v == vector(-2, 1)
        True

        """
        if self._hash is not None:
            raise ValueError('cannot rotate vector after hashing')
        radians = angle * math.pi / 180.0
        cosine = math.cos(radians)
        sine = math.sin(radians)
        x = self.x
        y = self.y
        self.x = x * cosine - y * sine
        self.y = y * cosine + x * sine

    def __repr__(self):
        """v.__repr__() -> repr(v)

        >>> v = vector(1, 2)
        >>> repr(v)
        'vector(1, 2)'

        """
        type_self = type(self)
        name = type_self.__name__
        return '{}({!r}, {!r})'.format(name, self.x, self.y)



import io
import os
import random
import runpy
import unittest.mock as mock


def test_main_list():
    random.seed(0)

    with mock.patch('sys.argv', ['__main__.py', 'list']):
        runpy.run_module('freegames.__main__')

def test_main_copy():
    random.seed(0)
    mock_open = mock.Mock()
    mock_open.side_effect = [io.StringIO(), io.StringIO()]

    with mock.patch('sys.argv', ['__main__.py', 'copy', 'guess']):
        with mock.patch('builtins.open', mock_open):
            runpy.run_module('freegames.__main__')

def test_main_copy_error():
    cwd = os.getcwd()
    path = os.path.join(cwd, 'guess.py')

    with open(path, 'w'):
        pass

    try:
        with mock.patch('sys.argv', ['__main__.py', 'copy', 'guess']):
            runpy.run_module('freegames.__main__')
    finally:
        os.remove('guess.py')

def test_main_show():
    random.seed(0)

    with mock.patch('sys.argv', ['__main__.py', 'show', 'guess']):
        runpy.run_module('freegames.__main__')



"""Mock turtle module.

"""

state = {}
events = []

class Turtle:
    def __init__(self, visible=True):
        pass

    def goto(self, x, y):
        pass

    def up(self):
        pass

    def down(self):
        pass

    def width(self, size):
        pass

    def color(self, fill, outline=None):
        pass

    def write(self, text, font=None):
        pass

    def begin_fill(self):
        pass

    def end_fill(self):
        pass

    def forward(self, steps):
        pass

    def back(self, steps):
        pass

    def addshape(self, reference):
        pass

    def shape(self, reference):
        pass

    def stamp(self):
        pass

    def left(self, degrees):
        pass

    def right(self, degrees):
        pass

    def hideturtle(self):
        pass

    def tracer(self, state):
        pass

    def dot(self, size, color=None):
        pass

    def clear(self):
        pass

    def circle(self, radius):
        pass

    def bgcolor(self, name):
        pass

    def update(self):
        pass

    def undo(self):
        pass

_turtle = Turtle()
goto = _turtle.goto
up = _turtle.up
down = _turtle.down
width = _turtle.width
color = _turtle.color
write = _turtle.write
begin_fill = _turtle.begin_fill
end_fill = _turtle.end_fill
forward = _turtle.forward
back = _turtle.back
addshape = _turtle.addshape
shape = _turtle.shape
stamp = _turtle.stamp
left = _turtle.left
right = _turtle.right
hideturtle = _turtle.hideturtle
tracer = _turtle.tracer
dot = _turtle.dot
clear = _turtle.clear
circle = _turtle.circle
bgcolor = _turtle.bgcolor
update = _turtle.update
undo = _turtle.undo

def setup(width, height, x, y):
    pass

def listen():
    pass

def onkey(function, key):
    state['key ' + key] = function

def ontimer(function, delay):
    state['timer'] = function
    state['delay'] = delay

def onscreenclick(function):
    state['click'] = function

def done():
    for event in events:
        name = event[0]

        try:
            function = state[name]
        except KeyError:
            if name == 'timer':
                if event[1:] and event[1]:
                    continue
            raise

        if name == 'timer':
            del state['timer']
            del state['delay']
            args = ()
        else:
            args = event[1:]

        function(*args)



import random
import runpy
import sys

import mockturtle

sys.modules['turtle'] = sys.modules['mockturtle']


def test_tiles():
    random.seed(0)
    mockturtle.events[:] = (
        ('click', x + 50, y + 50)
        for x in range(-200, 200, 100)
        for y in range(-200, 200, 100)
    )
    runpy.run_module('freegames.tiles')



import random
import runpy
import sys

import mockturtle

sys.modules['turtle'] = sys.modules['mockturtle']


def test_pong_1():
    random.seed(0)
    mockturtle.events[:] = (
        [('timer',), ('key s',)] * 8
        + [('timer', True)] * 600
    )
    runpy.run_module('freegames.pong')

def test_pong_2():
    random.seed(1)
    mockturtle.events[:] = (
        [('timer',), ('key k',)] * 12
        + [('timer', True)] * 600
    )
    runpy.run_module('freegames.pong')



"""Ant, simple animation demo.

Exercises

1. Wrap ant around screen boundaries.
2. Make the ant leave a trail.
3. Change the ant color based on position.
   Hint: colormode(255); color(0, 100, 200)

"""

from random import *
from turtle import *
from freegames import vector

ant = vector(0, 0)
aim = vector(2, 0)

def wrap(value):
    "Wrap value around -200 and 200."
    return value  # TODO

def draw():
    "Move ant and draw screen."
    ant.move(aim)
    ant.x = wrap(ant.x)
    ant.y = wrap(ant.y)

    aim.move(random() - 0.5)
    aim.rotate(random() * 10 - 5)

    clear()
    goto(ant.x, ant.y)
    dot(4)

    if running:
        ontimer(draw, 100)

setup(420, 420, 370, 0)
hideturtle()
tracer(False)
up()
running = True
draw()
done()



import random
import runpy
import unittest.mock as mock


def test_bagels_pass():
    random.seed(2)
    mock_input = mock.Mock()
    mock_input.side_effect = ['123', '1234', '456', '789', '810']
    mocks = {'print': lambda *args: None, 'input': mock_input}

    with mock.patch.multiple('builtins', **mocks):
        runpy.run_module('freegames.bagels')

def test_bagels_fail():
    random.seed(0)
    mock_input = mock.Mock()
    mock_input.side_effect = ['123'] * 11
    mocks = {'print': lambda *args: None, 'input': mock_input}

    with mock.patch.multiple('builtins', **mocks):
        runpy.run_module('freegames.bagels')



import random
import runpy
import sys

import mockturtle

sys.modules['turtle'] = sys.modules['mockturtle']


def test_paint():
    random.seed(0)
    mockturtle.events[:] = [
        ('key l',),
        ('click', 0, 0),
        ('click', 10, 10),
        ('key s',),
        ('click', 20, 20),
        ('click', 30, 30),
        ('key c',),
        ('click', 30, 30),
        ('click', 40, 40),
        ('key r',),
        ('click', 30, 30),
        ('click', 40, 40),
        ('key t',),
        ('click', 30, 30),
        ('click', 40, 40),
    ]
    runpy.run_module('freegames.paint')



"""Fidget, inspired by fidget spinners.

Exercises

1. Change the spinner pattern.
2. Respond to mouse clicks.
3. Change its acceleration.
4. Make it go forwards and backwards.

"""

from turtle import *

state = {'turn': 0}

def spinner():
    "Draw fidget spinner."
    clear()
    angle = state['turn'] / 10
    right(angle)
    forward(100)
    dot(120, 'red')
    back(100)
    right(120)
    forward(100)
    dot(120, 'green')
    back(100)
    right(120)
    forward(100)
    dot(120, 'blue')
    back(100)
    right(120)
    update()

def animate():
    "Animate fidget spinner."
    if state['turn'] > 0:
        state['turn'] -= 1

    spinner()
    ontimer(animate, 20)

def flick():
    "Flick fidget spinner."
    state['turn'] += 10

setup(420, 420, 370, 0)
hideturtle()
tracer(False)
width(20)
onkey(flick, 'space')
listen()
animate()
done()



"""Bounce, a simple animation demo.

Exercises

1. Make the ball speed up and down.
2. Change how the ball bounces when it hits a wall.
3. Make the ball leave a trail.
4. Change the ball color based on position.
   Hint: colormode(255); color(0, 100, 200)

"""

from random import *
from turtle import *
from freegames import vector

def value():
    "Randomly generate value between (-5, -3) or (3, 5)."
    return (3 + random() * 2) * choice([1, -1])

ball = vector(0, 0)
aim = vector(value(), value())

def draw():
    "Move ball and draw game."
    ball.move(aim)

    x = ball.x
    y = ball.y

    if x < -200 or x > 200:
        aim.x = -aim.x

    if y < -200 or y > 200:
        aim.y = -aim.y

    clear()
    goto(x, y)
    dot(10)

    ontimer(draw, 50)

setup(420, 420, 370, 0)
hideturtle()
tracer(False)
up()
draw()
done()
